@@ -17,25 +17,25 @@
 
 from warnings import warn
 from pyspark import keyword_only
-from ai.h2o.sparkling import Initializer
+from ai.h2o.sparkling.Initializer import Initializer
 from ai.h2o.sparkling.ml.Utils import Utils
 from ai.h2o.sparkling.ml.algos.H2OTreeBasedSupervisedAlgorithm import H2OTreeBasedSupervisedAlgorithm
 from ai.h2o.sparkling.ml.models.H2ODRFMOJOModel import H2ODRFMOJOModel
 from ai.h2o.sparkling.ml.params.H2ODRFParams import H2ODRFParams
-from ai.h2o.sparkling.ml.params.H2OCommonParams import H2OCommonParams
 from ai.h2o.sparkling.ml.algos.DistributionBasedH2OTrainFramePreparation import DistributionBasedH2OTrainFramePreparation
 
 
-class H2ODRF(H2ODRFParams, H2OCommonParams, H2OTreeBasedSupervisedAlgorithm, DistributionBasedH2OTrainFramePreparation):
+class H2ODRF(H2ODRFParams, H2OTreeBasedSupervisedAlgorithm, DistributionBasedH2OTrainFramePreparation):
 
     @keyword_only
     def __init__(self,
                  calibrationDataFrame=None,
                  ignoredCols=None,
                  columnsToCategorical=[],
+                 keepBinaryModels=False,
                  withContributions=False,
+                 dataFrameSerializer="ai.h2o.sparkling.utils.JSONDataFrameSerializer",
                  withLeafNodeAssignments=False,
-                 namedMojoOutputColumns=True,
                  convertInvalidNumbersToNa=False,
                  detailedPredictionCol="detailed_prediction",
                  validationDataFrame=None,
@@ -44,7 +44,6 @@ class H2ODRF(H2ODRFParams, H2OCommonParams, H2OTreeBasedSupervisedAlgorithm, Dis
                  convertUnknownCategoricalLevelsToNa=False,
                  splitRatio=1.0,
                  withStageResults=False,
-                 withDetailedPredictionCol=True,
                  mtries=-1,
                  binomialDoubleTrees=False,
                  sampleRate=0.632,
@@ -67,6 +66,7 @@ class H2ODRF(H2ODRFParams, H2OCommonParams, H2OTreeBasedSupervisedAlgorithm, Dis
                  minSplitImprovement=1.0E-5,
                  histogramType="AUTO",
                  calibrateModel=False,
+                 calibrationMethod="AUTO",
                  checkConstantResponse=True,
                  modelId=None,
                  nfolds=0,
@@ -88,12 +88,14 @@ class H2ODRF(H2ODRFParams, H2OCommonParams, H2OTreeBasedSupervisedAlgorithm, Dis
                  stoppingTolerance=0.001,
                  gainsliftBins=-1,
                  customMetricFunc=None,
-                 exportCheckpointsDir=None):
+                 exportCheckpointsDir=None,
+                 aucType="AUTO"):
         Initializer.load_sparkling_jar()
         super(H2ODRF, self).__init__()
         self._java_obj = self._new_java_obj("ai.h2o.sparkling.ml.algos.H2ODRF", self.uid)
         self._setDefaultValuesFromJava()
         kwargs = Utils.getInputKwargs(self)
+        kwargs = self._updateInitKwargs(kwargs)
 
         if 'interactionPairs' in kwargs:
             warn("Interaction pairs are not supported!")
