@@ -279,6 +279,18 @@ class H2OXGBoostParams(HasMonotoneConstraints, HasCalibrationDataFrame, HasIgnor
         """Controls the effect of observations with positive labels in relation to the observations with negative labels on gradient calculation. Useful for imbalanced problems.""",
         H2OTypeConverters.toFloat())
 
+    evalMetric = Param(
+        Params._dummy(),
+        "evalMetric",
+        """Specification of evaluation metric that will be passed to the native XGBoost backend.""",
+        H2OTypeConverters.toNullableString())
+
+    scoreEvalMetricOnly = Param(
+        Params._dummy(),
+        "scoreEvalMetricOnly",
+        """If enabled, score only the evaluation metric. This can make model training faster if scoring is frequent (eg. each iteration).""",
+        H2OTypeConverters.toBoolean())
+
     modelId = Param(
         Params._dummy(),
         "modelId",
@@ -309,6 +321,12 @@ class H2OXGBoostParams(HasMonotoneConstraints, HasCalibrationDataFrame, HasIgnor
         """Whether to keep the cross-validation fold assignment.""",
         H2OTypeConverters.toBoolean())
 
+    parallelizeCrossValidation = Param(
+        Params._dummy(),
+        "parallelizeCrossValidation",
+        """Allow parallel training of cross-validation models""",
+        H2OTypeConverters.toBoolean())
+
     distribution = Param(
         Params._dummy(),
         "distribution",
@@ -384,7 +402,7 @@ class H2OXGBoostParams(HasMonotoneConstraints, HasCalibrationDataFrame, HasIgnor
     stoppingMetric = Param(
         Params._dummy(),
         "stoppingMetric",
-        """Metric to use for early stopping (AUTO: logloss for classification, deviance for regression and anonomaly_score for Isolation Forest). Note that custom and custom_increasing can only be used in GBM and DRF with the Python client.""",
+        """Metric to use for early stopping (AUTO: logloss for classification, deviance for regression and anomaly_score for Isolation Forest). Note that custom and custom_increasing can only be used in GBM and DRF with the Python client.""",
         H2OTypeConverters.toEnumString("hex.ScoreKeeper$StoppingMetric"))
 
     stoppingTolerance = Param(
@@ -540,6 +558,12 @@ class H2OXGBoostParams(HasMonotoneConstraints, HasCalibrationDataFrame, HasIgnor
     def getScalePosWeight(self):
         return self.getOrDefault(self.scalePosWeight)
 
+    def getEvalMetric(self):
+        return self.getOrDefault(self.evalMetric)
+
+    def getScoreEvalMetricOnly(self):
+        return self.getOrDefault(self.scoreEvalMetricOnly)
+
     def getModelId(self):
         return self.getOrDefault(self.modelId)
 
@@ -555,6 +579,9 @@ class H2OXGBoostParams(HasMonotoneConstraints, HasCalibrationDataFrame, HasIgnor
     def getKeepCrossValidationFoldAssignment(self):
         return self.getOrDefault(self.keepCrossValidationFoldAssignment)
 
+    def getParallelizeCrossValidation(self):
+        return self.getOrDefault(self.parallelizeCrossValidation)
+
     def getDistribution(self):
         return self.getOrDefault(self.distribution)
 
@@ -735,6 +762,12 @@ class H2OXGBoostParams(HasMonotoneConstraints, HasCalibrationDataFrame, HasIgnor
     def setScalePosWeight(self, value):
         return self._set(scalePosWeight=value)
 
+    def setEvalMetric(self, value):
+        return self._set(evalMetric=value)
+
+    def setScoreEvalMetricOnly(self, value):
+        return self._set(scoreEvalMetricOnly=value)
+
     def setModelId(self, value):
         return self._set(modelId=value)
 
@@ -750,6 +783,9 @@ class H2OXGBoostParams(HasMonotoneConstraints, HasCalibrationDataFrame, HasIgnor
     def setKeepCrossValidationFoldAssignment(self, value):
         return self._set(keepCrossValidationFoldAssignment=value)
 
+    def setParallelizeCrossValidation(self, value):
+        return self._set(parallelizeCrossValidation=value)
+
     def setDistribution(self, value):
         return self._set(distribution=value)
 
